# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, clear pointing table, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.2.2', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.2.2')


# CALIBRATE_AMPLI: J0750+1231
# CALIBRATE_ATMOSPHERE: Ephemeris,J0750+1231,J0757+0956
# CALIBRATE_BANDPASS: J0750+1231
# CALIBRATE_FLUX: J0750+1231
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0757+0956
# CALIBRATE_POINTING: J0750+1231
# OBSERVE_TARGET: Ephemeris

# Using reference antenna = DV04

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_X90989d_X117c.ms') == False:
    importasdm('uid___A002_X90989d_X117c', asis='Antenna Station Receiver Source CalAtmosphere CalWVR')
  if applyonly != True: es.fixForCSV2555('uid___A002_X90989d_X117c.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_X90989d_X117c.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X90989d_X117c.ms.listobs')
  listobs(vis = 'uid___A002_X90989d_X117c.ms',
    listfile = 'uid___A002_X90989d_X117c.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X90989d_X117c.ms',
    mode = 'manual',
    spw = '1~12',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_X90989d_X117c.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_X90989d_X117c.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_X90989d_X117c.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_X90989d_X117c.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X90989d_X117c.ms.wvr') 
  
  os.system('rm -rf uid___A002_X90989d_X117c.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X90989d_X117c.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_X90989d_X117c.ms',
    caltable = 'uid___A002_X90989d_X117c.ms.wvr',
    toffset = 0,
    tie = ['Ephemeris,J0757+0956'],
    statsource = 'Ephemeris')
  
  casalog.setlogfile(mylogfile)
  
  # This is a temporary workaround, which will be included in a future version of CASA
  
  tb.open('uid___A002_X90989d_X117c.ms.wvr', nomodify=False)
  count = 0
  numrows = tb.nrows()
  mycparamcol = tb.getcol('CPARAM')
  for i in range(0, numrows):
      if mycparamcol[0][0][i] == (1.+0.j):
          tb.putcell('FLAG', i, [[True]])
          count += 1
  tb.close()
  del mycparamcol
  if(numrows>0):
      print 'Flagged', count, 'of', numrows, 'solutions =', 100.*count/float(numrows),'%'
  
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_X90989d_X117c.ms.wvr', spw='1', antenna='DV04',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_X90989d_X117c.ms.wvr.plots/uid___A002_X90989d_X117c.ms.wvr') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X90989d_X117c.ms.tsys') 
  gencal(vis = 'uid___A002_X90989d_X117c.ms',
    caltable = 'uid___A002_X90989d_X117c.ms.tsys',
    caltype = 'tsys')
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X90989d_X117c.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='5~123',showfdm=True, 
    field='', figfile='uid___A002_X90989d_X117c.ms.tsys.plots.overlayTime/uid___A002_X90989d_X117c.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_X90989d_X117c.ms.tsys', msName='uid___A002_X90989d_X117c.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: no baseline run found for antenna DV12.
  
  # Note: the correction for antenna DV19 is larger than 2mm.
  
  # Position for antenna DV19 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: the correction for antenna DA65 is larger than 2mm.
  
  # Position for antenna DA65 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: the correction for antenna DA64 is larger than 2mm.
  
  # Position for antenna DA64 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: the correction for antenna DA49 is larger than 2mm.
  
  # Position for antenna DA49 is derived from baseline run made on 2014-06-11 04:48:50.
  
  # Note: no baseline run found for antenna DA48.
  
  # Note: the correction for antenna DA60 is larger than 2mm.
  
  # Position for antenna DA60 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA45.
  
  # Note: no baseline run found for antenna DV13.
  
  # Note: the correction for antenna DA46 is larger than 2mm.
  
  # Position for antenna DA46 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA41.
  
  # Note: no baseline run found for antenna DV14.
  
  # Note: no baseline run found for antenna DA43.
  
  # Position for antenna DA42 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA63.
  
  # Note: no baseline run found for antenna DV15.
  
  # Note: no baseline run found for antenna DA62.
  
  # Note: no baseline run found for antenna DV08.
  
  # Note: no baseline run found for antenna DV11.
  
  # Note: the correction for antenna DA52 is larger than 2mm.
  
  # Position for antenna DA52 is derived from baseline run made on 2012-11-15 10:54:37.
  
  # Note: no baseline run found for antenna DA50.
  
  # Position for antenna DA51 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA56.
  
  # Position for antenna DV25 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA54.
  
  # Note: no baseline run found for antenna DV09.
  
  # Note: no baseline run found for antenna DV06.
  
  # Note: no baseline run found for antenna DV07.
  
  # Note: the correction for antenna DV04 is larger than 2mm.
  
  # Position for antenna DV04 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DV17.
  
  os.system('rm -rf uid___A002_X90989d_X117c.ms.antpos') 
  gencal(vis = 'uid___A002_X90989d_X117c.ms',
    caltable = 'uid___A002_X90989d_X117c.ms.antpos',
    caltype = 'antpos',
    antenna = 'DV19,DA52,DA65,DA64,DA49,DV25,DA60,DV04,DA46,DA42,DA51',
    parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
  #  parameter = [-0.00414290977642,0.0092498883605,0.00376623217016,0.000253285747021,0.00267631281167,0.00169306434691,-0.0019226311706,0.00491832382977,0.00211138557643,-0.00410456955433,0.00726661179215,0.00325340544805,-0.000611576717347,0.00173676759005,0.00119661167264,-0.000981642864645,0.00105158798397,0.000677143689245,-0.00318624591455,0.00662772543728,0.00263020861894,-0.00236665643752,0.00325250066817,0.000717006158084,-0.00178120005876,0.00230418425053,0.000323298387229,-0.000526665726542,0.000489968824986,0.000222492163967,-0.000559699778247,0.000879617942017,0.000171634595327])
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X90989d_X117c.ms', tsystable = 'uid___A002_X90989d_X117c.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_X90989d_X117c.ms',
    field = '0',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X90989d_X117c.ms.tsys', 'uid___A002_X90989d_X117c.ms.wvr', 'uid___A002_X90989d_X117c.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X90989d_X117c.ms',
    field = '1',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X90989d_X117c.ms.tsys', 'uid___A002_X90989d_X117c.ms.wvr', 'uid___A002_X90989d_X117c.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X90989d_X117c.ms',
    field = '2',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X90989d_X117c.ms.tsys', 'uid___A002_X90989d_X117c.ms.wvr', 'uid___A002_X90989d_X117c.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_X90989d_X117c.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X90989d_X117c.ms.split') 
  split(vis = 'uid___A002_X90989d_X117c.ms',
    outputvis = 'uid___A002_X90989d_X117c.ms.split',
    datacolumn = 'corrected',
    spw = '1,3,5,7',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.listobs')
  listobs(vis = 'uid___A002_X90989d_X117c.ms.split',
    listfile = 'uid___A002_X90989d_X117c.ms.split.listobs')
  
  tb.open('uid___A002_X90989d_X117c.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  
  if not os.path.exists('uid___A002_X90989d_X117c.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_X90989d_X117c.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X90989d_X117c.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_X90989d_X117c.ms.split',
    mode = 'manual',
    spw = '0:0~7;120~127,1:0~7;120~127,2:0~7;120~127,3:0~7;120~127',
    flagbackup = F)
  flagdata(vis = 'uid___A002_X90989d_X117c.ms.split',
    mode = 'manual',
    antenna = 'DV06,DA45,DV19', # low gain (worst in spw0), low gain, bad pointing
    flagbackup = F)
  
  

# Putting a model for the flux calibrator(s)
mystep = 11
refant = 'DV09'
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_X90989d_X117c.ms.split',
    field = '0', # source name = J0750+1231
    spix = -0.714, reffreq = '233.0GHz',
    standard = 'manual',
    fluxdensity = [0.639, 0, 0, 0]) 
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X90989d_X117c.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_X90989d_X117c.ms.split',
    caltable = 'uid___A002_X90989d_X117c.ms.split.ap_pre_bandpass',
    field = '0', # J0750+1231
    spw = '0:51~76,1:51~76,2:51~76,3:51~76',
    solint = 'int',
    refant = refant,
    calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_X90989d_X117c.ms.split.ap_pre_bandpass', msName='uid___A002_X90989d_X117c.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X90989d_X117c.ms.split',
    caltable = 'uid___A002_X90989d_X117c.ms.split.bandpass',
    field = '0', # J0750+1231
    solint = 'inf',
    combine = 'scan',
    refant = refant,
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_X90989d_X117c.ms.split.ap_pre_bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X90989d_X117c.ms.split.bandpass', msName='uid___A002_X90989d_X117c.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X90989d_X117c.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_X90989d_X117c.ms.split',
    caltable = 'uid___A002_X90989d_X117c.ms.split.phase_int',
    field = '0~1', # J0750+1231,J0757+0956
    solint = 'int',
    refant = refant,
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X90989d_X117c.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X90989d_X117c.ms.split.phase_int', msName='uid___A002_X90989d_X117c.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_X90989d_X117c.ms.split',
    caltable = 'uid___A002_X90989d_X117c.ms.split.ampli_inf',
    field = '0~1', # J0750+1231,J0757+0956
    solint = 'inf',
    refant = refant,
    gaintype = 'T',
    calmode = 'a',
    gaintable = ['uid___A002_X90989d_X117c.ms.split.bandpass', 'uid___A002_X90989d_X117c.ms.split.phase_int'])
  
  if applyonly != True: es.checkCalTable('uid___A002_X90989d_X117c.ms.split.ampli_inf', msName='uid___A002_X90989d_X117c.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X90989d_X117c.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_X90989d_X117c.ms.split',
    caltable = 'uid___A002_X90989d_X117c.ms.split.ampli_inf',
    fluxtable = 'uid___A002_X90989d_X117c.ms.split.flux_inf',
    reference = '0') # J0750+1231
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_X90989d_X117c.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_X90989d_X117c.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X90989d_X117c.ms.split',
    caltable = 'uid___A002_X90989d_X117c.ms.split.phase_inf',
    field = '0~1', # J0750+1231,J0757+0956
    solint = 'inf',
    refant = refant,
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X90989d_X117c.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X90989d_X117c.ms.split.phase_inf', msName='uid___A002_X90989d_X117c.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X90989d_X117c.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0']: # J0750+1231
    applycal(vis = 'uid___A002_X90989d_X117c.ms.split',
      field = str(i),
      gaintable = ['uid___A002_X90989d_X117c.ms.split.bandpass', 'uid___A002_X90989d_X117c.ms.split.phase_int', 'uid___A002_X90989d_X117c.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = F,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_X90989d_X117c.ms.split',
    field = '1,2', # Ephemeris
    gaintable = ['uid___A002_X90989d_X117c.ms.split.bandpass', 'uid___A002_X90989d_X117c.ms.split.phase_inf', 'uid___A002_X90989d_X117c.ms.split.flux_inf'],
    gainfield = ['', '1', '1'], # J0757+0956
    interp = 'linear,linear',
    calwt = F,
    flagbackup = F)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

#  os.system('rm -rf uid___A002_X90989d_X117c.ms.split.cal') 
#  split(vis = 'uid___A002_X90989d_X117c.ms.split',
#    outputvis = 'uid___A002_X90989d_X117c.ms.split.cal',
#    datacolumn = 'corrected',
#    keepflags = T)
  set1='9~35'
  set2='37~67'
  os.system('rm -rf Juno6.ms.cal') 
  split(vis = 'uid___A002_X90989d_X117c.ms.split', field='2',scan=set1,
    outputvis = 'Juno6.ms.cal',width=128,
    datacolumn = 'corrected',
    keepflags = T)
  os.system('rm -rf Juno7.ms.cal') 
  split(vis = 'uid___A002_X90989d_X117c.ms.split', field='2',scan=set2,
    outputvis = 'Juno7.ms.cal',width=128,
    datacolumn = 'corrected',
    keepflags = T)
  
  

